@@ -18,6 +18,118 @@ from litepipeline.manager.config import CONFIG
 LOG = logging.getLogger(__name__)
 
 
+BaseVenvs = declarative_base()
+
+class VenvsTable(BaseVenvs):
+    __tablename__ = "venvs"
+
+    id = Column(Integer, primary_key = True, autoincrement = True)
+    venv_id = Column(Text, unique = True, nullable = False, index = True)
+    name = Column(Text, nullable = False, index = True)
+    create_at = Column(DateTime, nullable = False, index = True)
+    update_at = Column(DateTime, nullable = False, index = True)
+    sha1 = Column(Text, nullable = False)
+    description = Column(Text)
+
+    @classmethod
+    def init_engine_and_session(cls):
+        cls.engine = create_engine('sqlite:///' + os.path.join(CONFIG["data_path"], "venvs.db"), echo = False)
+        cls.session = sessionmaker(bind = cls.engine)
+        return cls.engine, cls.session
+
+    def to_dict(self):
+        return {
+            "id": self.id,
+            "venv_id": self.venv_id,
+            "name": self.name,
+            "create_at": str(self.create_at), # "%Y-%m-%d %H:%M:%S.%f")
+            "update_at": str(self.update_at),
+            "sha1": self.sha1,
+            "description": self.description,
+        }
+
+    def parse_dict(self, source):
+        result = False
+
+        attrs = [
+            "venv_id",
+            "name",
+            "create_at",
+            "update_at",
+            "sha1",
+            "description",
+        ]
+
+        if hasattr(source, "__getitem__"):
+            for attr in attrs:
+                try:
+                    setattr(self, attr, source[attr])
+                except:
+                    LOG.debug("some exception occured when extract %s attribute to object, i will discard it",
+                        attr)
+                    continue
+            result = True
+        else:
+            LOG.debug("input param source does not have dict-like method, so i will do nothing at all!")
+        return result
+
+    def __repr__(self):
+        return "venv: %s" % self.to_dict()
+
+
+BaseVenvHistory = declarative_base()
+
+class VenvHistoryTable(BaseVenvHistory):
+    __tablename__ = "venv_history"
+
+    id = Column(Integer, primary_key = True, autoincrement = True)
+    venv_id = Column(Text, nullable = False, index = True)
+    create_at = Column(DateTime, nullable = False, index = True)
+    sha1 = Column(Text, nullable = False, index = True)
+    description = Column(Text)
+
+    @classmethod
+    def init_engine_and_session(cls):
+        cls.engine = create_engine('sqlite:///' + os.path.join(CONFIG["data_path"], "venv_history.db"), echo = False)
+        cls.session = sessionmaker(bind = cls.engine)
+        return cls.engine, cls.session
+
+    def to_dict(self):
+        return {
+            "id": self.id,
+            "venv_id": self.venv_id,
+            "create_at": str(self.create_at), # "%Y-%m-%d %H:%M:%S.%f")
+            "sha1": self.sha1,
+            "description": self.description,
+        }
+
+    def parse_dict(self, source):
+        result = False
+
+        attrs = [
+            "venv_id",
+            "create_at",
+            "sha1",
+            "description",
+        ]
+
+        if hasattr(source, "__getitem__"):
+            for attr in attrs:
+                try:
+                    setattr(self, attr, source[attr])
+                except:
+                    LOG.debug("some exception occured when extract %s attribute to object, i will discard it",
+                        attr)
+                    continue
+            result = True
+        else:
+            LOG.debug("input param source does not have dict-like method, so i will do nothing at all!")
+        return result
+
+    def __repr__(self):
+        return "venv_history: %s" % self.to_dict()
+
+
 BaseApplications = declarative_base()
 
 class ApplicationsTable(BaseApplications):
@@ -140,6 +252,7 @@ class TasksTable(BaseTasks):
     task_name = Column(Text, nullable = False, index = True)
     application_id = Column(Text, nullable = False, index = True)
     work_id = Column(Text, nullable = False, index = True)
+    service_id = Column(Text, nullable = False, index = True)
     create_at = Column(DateTime, nullable = False, index = True)
     start_at = Column(DateTime, index = True)
     update_at = Column(DateTime, nullable = False, index = True)
@@ -162,6 +275,7 @@ class TasksTable(BaseTasks):
             "task_name": self.task_name,
             "application_id": self.application_id,
             "work_id": self.work_id,
+            "service_id": self.service_id,
             "create_at": str(self.create_at),
             "start_at": str(self.start_at) if self.start_at else self.start_at,
             "update_at": str(self.update_at),
@@ -180,6 +294,7 @@ class TasksTable(BaseTasks):
             "task_name",
             "application_id",
             "work_id",
+            "service_id",
             "create_at",
             "start_at",
             "update_at",
@@ -421,3 +536,80 @@ class SchedulesTable(BaseSchedules):
 
     def __repr__(self):
         return "schedule: %s" % self.to_dict()
+
+
+BaseServices = declarative_base()
+
+class ServicesTable(BaseServices):
+    __tablename__ = "services"
+
+    id = Column(Integer, primary_key = True, autoincrement = True)
+    service_id = Column(Text, unique = True, nullable = False, index = True)
+    name = Column(Text, nullable = False, index = True)
+    application_id = Column(Text, nullable = False, index = True)
+    task_id = Column(Text, nullable = False, index = True)
+    create_at = Column(DateTime, nullable = False, index = True)
+    update_at = Column(DateTime, nullable = False, index = True)
+    stage = Column(Text, nullable = False, index = True)
+    status = Column(Text, index = True)
+    input_data = Column(Text)
+    description = Column(Text)
+    signal = Column(Integer, nullable = True) # -9, -15
+    enable = Column(Boolean, nullable = False)
+
+    @classmethod
+    def init_engine_and_session(cls):
+        cls.engine = create_engine('sqlite:///' + os.path.join(CONFIG["data_path"], "services.db"), echo = False)
+        cls.session = sessionmaker(bind = cls.engine)
+        return cls.engine, cls.session
+
+    def to_dict(self):
+        return {
+            "id": self.id,
+            "service_id": self.service_id,
+            "name": self.name,
+            "application_id": self.application_id,
+            "task_id": self.task_id,
+            "create_at": str(self.create_at),
+            "update_at": str(self.update_at),
+            "stage": self.stage,
+            "status": self.status,
+            "input_data": json.loads(self.input_data),
+            "description": self.description,
+            "signal": self.signal,
+            "enable": self.enable,
+        }
+
+    def parse_dict(self, source):
+        result = False
+
+        attrs = [
+            "service_id",
+            "name",
+            "application_id",
+            "task_id",
+            "create_at",
+            "update_at",
+            "stage",
+            "status",
+            "input_data",
+            "description",
+            "signal",
+            "enable",
+        ]
+
+        if hasattr(source, "__getitem__"):
+            for attr in attrs:
+                try:
+                    setattr(self, attr, source[attr])
+                except:
+                    LOG.debug("some exception occured when extract %s attribute to object, i will discard it",
+                        attr)
+                    continue
+            result = True
+        else:
+            LOG.debug("input param source does not have dict-like method, so i will do nothing at all!")
+        return result
+
+    def __repr__(self):
+        return "service: %s" % self.to_dict()
