@@ -18,6 +18,7 @@ function taskInit (manager_host) {
     var filter_value = "";
     var current_page = 1;
     var current_page_size = 50;
+    var host = window.location.host;
 
     getTaskList();
     $btn_refresh.bind('click', refreshPage);
@@ -84,6 +85,7 @@ function taskInit (manager_host) {
                 $table_header_tr.append(getHeaderTR('task_name', 'name', 'name'));
                 $table_header_tr.append(getHeaderTR('application_id', 'application id', 'application id'));
                 $table_header_tr.append(getHeaderTR('work_id', 'work id', 'work id'));
+                $table_header_tr.append(getHeaderTR('service_id', 'service id', 'service id'));
                 $table_header_tr.append(getHeaderTR('create_at', 'create at', 'create at'));
                 $table_header_tr.append(getHeaderTR('start_at', 'start at', 'start at'));
                 $table_header_tr.append(getHeaderTR('end_at', 'end at', 'end at'));
@@ -96,6 +98,7 @@ function taskInit (manager_host) {
                     "task_name",
                     "application_id",
                     "work_id",
+                    "service_id",
                     "create_at",
                     "start_at",
                     "end_at",
@@ -120,7 +123,7 @@ function taskInit (manager_host) {
                             tr += '<button id="' + value["task_id"] + '" type="button" class="btn btn-secondary btn-sm btn-operation btn-delete" onclick="this.blur();"><span class="oi oi-circle-x" title="delete" aria-hidden="true"></span></button>';
                             tr += '<button id="' + value["task_id"] + '" type="button" class="btn btn-secondary btn-sm btn-operation btn-detail" onclick="this.blur();"><span class="oi oi-spreadsheet" title="detail" aria-hidden="true"></span></button>';
                             tr += '</div></div></td>';
-                        } else if (col == 'task_id' || col == 'application_id' || col == 'work_id') {
+                        } else if (col == 'task_id' || col == 'application_id' || col == 'work_id' || col == 'service_id') {
                             tr += '<td id="' + col + '"><div class="outer"><div class="inner"><span class="span-pre">' + value[col] + '</span></div></div></td>';
                         } else if (col == 'task_name') {
                             tr += '<td id="' + col + '" title="' + value[col] + '"><div class="outer"><div class="inner">&nbsp;' + value[col] + '</div></div></td>';
@@ -146,7 +149,7 @@ function taskInit (manager_host) {
                 $(".btn-stop").bind('click', showTaskStop);
                 $(".btn-download").bind('click', showTaskDownload);
                 $(".btn-delete").bind('click', showTaskDelete);
-                $(".btn-detail").bind('click', showTaskDetail);
+                $(".btn-detail").bind('click', openTaskInfoPage);
 
                 if (task_id) {
                     var info = {};
@@ -256,7 +259,7 @@ function taskInit (manager_host) {
     }
 
     async function stopTask() {
-        var data = {"task_id": current_task_id, "signal": -9};
+        var data = {"task_id": current_task_id, "signal": Number($('#form_stop select#stop_signal').val())};
         $('#task_stop_modal').modal('hide');
         showWaitScreen();
         await sleep(1000);
@@ -282,13 +285,35 @@ function taskInit (manager_host) {
     function showTaskDownload() {
         current_task_id = $(this).attr("id");
         $("select#action_name").empty();
-        var actions = task_info[current_task_id].result;
-        for (var action_name in actions) {
-            $("select#action_name").append(
-                '<option value="' + action_name + '">' + action_name + '</option>'
-            );
-        }
-        $('#task_download_modal').modal('show');
+        var url = "http://" + manager_host + "/task/info?task_id=" + current_task_id;
+        $.ajax({
+            dataType: "json",
+            url: url,
+            success: function(data) {
+                if (data.result != "ok") {
+                    showWarningToast("operation failed", data.message);
+                } else {
+                    for (var action_name in data.task_info.result) {
+                        $("select#action_name").append(
+                            '<option value="' + action_name + '">' + action_name + '</option>'
+                        );
+                    }
+                    if (data.task_running_info) {
+                        for (var i = 0; i < data.task_running_info.length; i++) {
+                            $("select#action_name").append(
+                                '<option value="' + data.task_running_info[i].name + '">' + data.task_running_info[i].name + '</option>'
+                            );
+                        }
+                    }
+                    $('#task_download_modal').modal('show');
+                }
+                hideWaitScreen();
+            },
+            error: function() {
+                showWarningToast("error", "request service failed");
+                hideWaitScreen();
+            }
+        });
     }
 
     async function downloadTask() {
@@ -382,11 +407,11 @@ function taskInit (manager_host) {
         });
     }
 
-    function showTaskDetail() {
+    function openTaskInfoPage() {
         var task_id = $(this).attr("id");
-        document.getElementById("task_info_json").textContent = JSON.stringify(task_info[task_id], undefined, 4);
-        $('#task_info_refresh').bind('click', {"task_id": task_id}, refreshTaskInfo);
-        $('#task_info_modal').modal('show');
+        var url = "http://" + host + "/task/" + task_id;
+        var win = window.open(url, '_blank');
+        win.focus();
     }
 
     function changePage() {
@@ -410,6 +435,7 @@ function taskInit (manager_host) {
     function resetModal(e) {
         $("#" + e.target.id).find("input:text").val("");
         $("#" + e.target.id).find("textarea").val("");
+        $("#" + e.target.id).find('select#stop_signal').val("-9");
     }
 
     function addColumnsCSS(keys) {
@@ -449,6 +475,11 @@ function taskInit (manager_host) {
             $('td#work_id').css("width", "10%");
             percent -= 10.0;
         }
+        if (is_in('service_id', keys)) {
+            $('th#service_id').css("width", "10%");
+            $('td#service_id').css("width", "10%");
+            percent -= 10.0;
+        }
         if (is_in('stage', keys)) {
             $('th#stage').css("width", "5%");
             $('td#stage').css("width", "5%");
